dnl
dnl MOD_SETUP - Create a valid module environment
dnl
AC_DEFUN([MOD_SETUP],[
	AC_MSG_CHECKING([for current modules list])
	if test -f modules-current.list; then
		AC_MSG_RESULT(found)
	else
		cp modules.list modules-current.list
		AC_MSG_RESULT(created)
	fi

	AC_MSG_CHECKING([for all static])
	AC_ARG_ENABLE([all-static],
[  --enable-all-static     Make all modules compile statically],[
	for fil in `cat modules-current.list`;
	do
		mod_name=`echo "$fil" | sed 's/\(.*\)-\(.*\)-\(.*\)/\1/'`
		MOD_SET_COMP($mod_name,static)
	done
	AC_MSG_RESULT([yes])
])
	AC_MSG_RESULT([no])
])

dnl
dnl MOD_SET_COMP - Set a module to a certain type, usually dso or static
dnl      1)  Module name
dnl      2)  How to compile (dso,static)
dnl
AC_DEFUN([MOD_SET_COMP],[
	fil_list=`cat modules-current.list`
	rm modules-current.list
	touch modules-current.list
	for fil in $fil_list;
	do
		mod_name=`echo "$fil" | sed 's/\(.*\)-\(.*\)-\(.*\)/\1/'`
		mod_type=`echo "$fil" | sed 's/\(.*\)-\(.*\)-\(.*\)/\2/'`
		mod_comp=`echo "$fil" | sed 's/\(.*\)-\(.*\)-\(.*\)/\3/'`
		if test x$mod_name = x$1; then
			echo "$mod_name-$mod_type-$2" >> modules-current.list;
		else
			echo "$mod_name-$mod_type-$mod_comp" >> modules-current.list
		fi
	done
])

dnl
dnl MOD_GEN - Generate build variables
dnl
AC_DEFUN([MOD_GEN],[
	AC_SUBST(MOD_DSOS)
	AC_SUBST(MOD_STATICS)
	AC_SUBST(MOD_STSRC)
	AC_SUBST(MOD_FUNCS)
	fil_list=`cat modules-current.list`
	for fil in $fil_list;
	do
		mod_name=`echo "$fil" | sed 's/\(.*\)-\(.*\)-\(.*\)/\1/'`
		mod_type=`echo "$fil" | sed 's/\(.*\)-\(.*\)-\(.*\)/\2/'`
		mod_comp=`echo "$fil" | sed 's/\(.*\)-\(.*\)-\(.*\)/\3/'`
		if test x$mod_type = xmodule; then
			if test x$mod_comp = xstatic; then
				MOD_STATICS=" modules/mod_$mod_name.o $MOD_STATICS"
				MOD_STSRC=" mod_$mod_name.c $MOD_STSRC"
				MOD_FUNCS="mod_$mod_name(); $MOD_FUNCS "
			else
				MOD_DSOS=" mod_$mod_name.so $MOD_DSOS"
			fi
		fi
	done
])
