#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <jabber/jabber.h>
#include <etherx.h>

#define HASH_PRIME 509 /* set to a prime number larger then the average max # of users, for the master hash */

#define MAPI_VARAUTH -10
#define MAPI_VARREGISTER -11

typedef enum {C_SET, C_GET, C_DEL, C_CHECK, C_INIT} command;

typedef enum {S_BOUNCE, S_IGNORE, S_AUTHED} sreturn;

typedef enum {P_SESSION, P_UNKNOWN, P_OFFLINE, P_SERVER, P_DELIVER, P_SHUTDOWN, PS_IN, PS_OUT, PS_END} mphase;
typedef enum {M_PASS, M_IGNORE, M_HANDLED} mreturn;


typedef struct udata_struct *udata, _udata;
typedef struct session_struct *session, _session;

typedef struct mapi_struct
{
    jpacket packet;
    mphase phase;
    udata user;
    session s;
    int variant;
} *mapi, _mapi;

typedef mreturn (*mcall)(mapi m, void *arg);

typedef struct mlist_struct
{
    mcall c;
    void *arg;
    unsigned char mask;
    struct mlist_struct *next;
} *mlist, _mlist;

/* contains a list of module function pointers */
typedef struct mapi_master
{
    mphase p;
    mlist l;
    struct mapi_master *next;
} *mmaster, _mmaster;

mmaster js_mapi_master(mphase p);

typedef struct xdb_struct
{
    char *ns;
    xmlnode data;
    int cache;
    struct xdb_struct *next;
} *xdb, _xdb;

typedef int (*xcall)(int set, xdb x, udata u, void *arg);

/* contains a list of xdb handler function pointers */
typedef struct xlist_struct
{
    xcall c;
    void *arg;
    struct xlist_struct *next;
} *xlist, _xlist;

void js_xdb_register(xcall c, void *arg);
xmlnode js_xdb_get(udata user, char *ns);
void js_xdb_set(udata user, char *ns, xmlnode x);

void js_service_listen(char *svc, tlisten_onConnect evt);

ehandler js_names(int cmd, char *name);


struct udata_struct
{
    char *user;
    session sessions;
    int scount, ref;
    xdb x_cache;
    ppdb p_cache;
    rlimit rate;
    pool p;
    struct udata_struct *next;
};

xmlnode js_config(char *query);

extern HASHTABLE js__users;
udata js_user(char *user);
void js_users_exit(void);
void js_deliver(jpacket p);


typedef void (*session_onSend)(session s, jpacket p, void *arg);

struct session_struct
{
    /* general session data */
    char *res;
    jid id, uid;
    udata u;
    xmlnode presence;
    int priority, roster;
    int c_in, c_out;
    time_t started;

    /* mechanics */
    pool p;
    int exit_flag;
    mlist m_in, m_out, m_end;
    pth_msgport_t worker;

    /* send handler */
    session_onSend send;
    void *arg;

    struct session_struct *next;
};

session js_session_new(jid owner, session_onSend send, void *arg);
void *js_session_main(void *arg);
void js_session_end(session s, char *reason);
session js_session_get(udata user, char *res);
session js_session_primary(udata user);
void js_session_to(session s, jpacket p);
void js_session_from(session s, jpacket p);

void js_conn_connect(thread t, int sock, struct sockaddr_in sa);
void js_conn_negotiate(thread t, xmlnode in, int type);
xmlstream_onNode js_conn_namespace(command cmd, char *ns, xmlstream_onNode handler);

void *js_server_main(void *arg);
void *js_offline_main(void *arg);
void *js_unknown_main(void *arg);
void *js_users_main(void *arg);
void *js_debug_main(void *arg);

typedef struct {
    pth_message_t head; /* the standard pth message header */
    jpacket p;
} _jpq, *jpq;

void js_psend(pth_msgport_t mp, jpacket p); /* sends p to a pth message port */

void js_bounce(xmlnode x, terror terr); /* logic to bounce packets w/o looping, eats x and delivers error */

int js_config_name(command cmd, char *name);
int js_config_load(char *file); /* load the config file */
extern char *js__hostname; /* server name */
extern xmlnode js__config; /* loaded server config */

sreturn js_service_prescreen(jpacket p);
void js_mapi_register(mphase p, mcall c, void *arg);
void js_mapi_session(mphase p, session s, mcall c, void *arg);
int js_mapi_call(mphase phase, mlist l, jpacket packet, udata user, session s, int variant);

void js_static(void);

