/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 *
 *  mod_dso.c -- loads and initializes dynamic modules
 *
 */

#include "jserver.h"
#include "dlfcn.h"

/*
 *  mod_dso_load -- loads the shared object and calls the init function
 *
 *  parameters
 *  	so -- the name of the shared object
 *      init -- the name of the init function
 */
void mod_dso_load(char *so, char *init)
{
    void (*init_h)(void);	/* a pointer to the dso's init function */
    void *so_h;				/* a pointer to the object code in memory */
    char *dlerr;			/* error message */

    /* ignore illegal calls */
    if(so == NULL || init == NULL) return;

    /* debug message */
    log_debug(ZONE,"mod_dso_load %s",so);

    /* load the dso */
    so_h = dlopen(so,RTLD_LAZY);

    /* check for a load error */
    dlerr = dlerror();
    if(dlerr != NULL)
    {
        log_error("jserver","mod_dso loading %s failed: %s",so,dlerr);
        return;
    }

    /* resolve a reference to the dso's init function */
    init_h = dlsym(so_h,init);

    /* check for error */
    dlerr = dlerror();
    if(dlerr != NULL)
    {
        log_error("jserver","mod_dso calling %s in %s failed: %s",init,so,dlerr);
        dlclose(so_h);
        return;
    }

    /* call the init function */
    (init_h)();

}


/*
 *  mod_dso -- loads the dso components listed in the config
 */
void mod_dso(void)
{
    xmlnode cfg;	/* xml node from the configuration file */

    /* debug message */
    log_debug(ZONE,"mod_dso");

    /* fetch the dso config node */
    cfg = js_config("dso");

    /* if no dsos are configured return */
    if(cfg == NULL)
        return;

    /* scan through the dso's specified */
    for(cfg = xmlnode_get_firstchild(cfg); cfg != NULL; cfg = xmlnode_get_nextsibling(cfg))

        /* if this is the name of a dso... */
        if(xmlnode_get_type(cfg) == NTYPE_TAG)

            /* ... load it! */
            mod_dso_load(xmlnode_get_data(cfg), xmlnode_get_name(cfg));

}
