#include "jserver.h"

#ifdef HAVE_OPENSSL
#include "svc_client_ssl.h"

static SSL_CTX *ctx = NULL;
static RSA *rsa_tmp = NULL;

static void print_cipher(SSL *ssl) /* print negotiated cipher */
{
    SSL_CIPHER *c;
    char *ver;
    int bits;

    switch(ssl->session->ssl_version) {
    case SSL2_VERSION:
        ver="SSLv2"; break;
    case SSL3_VERSION:
        ver="SSLv3"; break;
    case TLS1_VERSION:
        ver="TLSv1"; break;
    default:
        ver="UNKNOWN";
    }
    c=SSL_get_current_cipher(ssl);
    SSL_CIPHER_get_bits(c, &bits);
    log_debug(ZONE, "connection opened with %s, cipher %s (%u bits)",
              ver, SSL_CIPHER_get_name(c), bits);
}

int svc_ssl_SSL_write(tstream ts, const void *buf, size_t count)
{
    return SSL_write((SSL *)ts->data_aux4, buf, count);
}

int svc_ssl_SSL_read(tstream ts, const void *buf, size_t count)
{
    return SSL_read((SSL *)ts->data_aux4, (char *)buf, count);
}

void svc_ssl_connect(thread t, int sock, struct sockaddr_in sa)
{
    /* XXX setup the SSL stuff and put is in it */
    char *cert_file=NULL;
    SSL *ssl=NULL;
    thread new;
    tstream ts;

    log_debug(ZONE, "svc_ssl:  New connection");

    if(ctx == NULL)
    {
        SSLeay_add_ssl_algorithms();
        SSL_load_error_strings();

        ctx=SSL_CTX_new(SSLv23_server_method());

        /* Insert obligatory grumble about patents */
#ifndef NO_RSA
        //log(LOG_DEBUG, "Generating %d bit temporary RSA key...", KEYLENGTH);
        log_debug(ZONE, "Generating %d bit temporary RSA key...", KEYLENGTH);
        rsa_tmp=RSA_generate_key(KEYLENGTH, RSA_F4, NULL, NULL);
        if(!rsa_tmp) {
            //sslerror("tmp_rsa_cb");
            log_debug(ZONE, "Error setting tmp_rsa_cb");
            exit(1);
        }
        //log(LOG_DEBUG, "Temporary RSA key generated");
        log_debug(ZONE, "Temporary RSA key generated\n");
        // XXX Please someone explain...
        //SSL_CTX_set_tmp_rsa_callback(ctx, tmp_rsa_cb);
#endif /* NO_RSA */

        /* Need to read docs to figure out what this actually does */
        SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_BOTH);
        /* Should we use a default timeout value?  configured? */
        //SSL_CTX_set_timeout(ctx, options.session_timeout);
        cert_file = xmlnode_get_data(js_config("cert"));
        if(cert_file) {
            if(!SSL_CTX_use_certificate_file(ctx, cert_file,
                                             SSL_FILETYPE_PEM)) {
                log_error(ZONE, "Error reading certificate file: %s", cert_file);
                //sslerror("SSL_CTX_use_certificate_file");
                exit(1);
            }
            log_debug(ZONE, "Certificate: %s", cert_file);
#ifdef NO_RSA
            if(!SSL_CTX_use_PrivateKey_file(ctx, cert_file,
                                            SSL_FILETYPE_PEM)) {
                //sslerror("SSL_CTX_use_PrivateKey_file");
                exit(1);
            }
#else /* NO_RSA */
if(!SSL_CTX_use_RSAPrivateKey_file(ctx, cert_file,
            SSL_FILETYPE_PEM)) {
                //sslerror("SSL_CTX_use_RSAPrivateKey_file");
                exit(1);
            }
#endif /* NO_RSA */
        }
        /* Better read docs for this one, I don't know yet XXX */
        //SSL_CTX_set_info_callback(ctx, info_callback);

        /* I want to decide how to pass this around first */
#if 0
        if(options.cipher_list) {
            if (!SSL_CTX_set_cipher_list(ctx, options.cipher_list)) {
                sslerror("SSL_CTX_set_cipher_list");
                exit(1);
            }
        }
#endif
    }

    if(ctx == NULL)
        log_debug(ZONE, "We have a (NULL) ctx!!");
    ssl = SSL_new(ctx);
    SSL_set_fd(ssl, sock);
    SSL_set_accept_state(ssl);
    if(SSL_accept(ssl) <= 0){
        log_debug(ZONE, "SSL Error in SSL_accept call");
        SSL_free(ssl);
        close(sock);
        return;
    }

    print_cipher(ssl);

    new = xmlstream_new(sock, inet_ntoa(sa.sin_addr),
                        (xmlstream_onNode)js_conn_negotiate);
    ts = (tstream)(new->data);
    tstream_rw(ts, svc_ssl_SSL_read, svc_ssl_SSL_write);
    ts->data_aux4 = (void *)ssl;

    return;
}
#endif /* HAVE_OPENSSL */
