#include "jserver.h"

#define SVC_HELL_MAX 100
#define SVC_HELL_ITER 10000

typedef struct svc_hell_data_struct
{
    session s;
    char id[10];
} helld;

helld hell[SVC_HELL_MAX];
int svc_hell_sleep = 1;
int svc_hell_total = 0;

void svc_hell_onSend(session s, jpacket p, void *arg)
{
    int demon = (int)arg;

    if(p == NULL)
    { /* signal to exit from the session */
        fprintf(stderr,".");
        hell[demon].s = NULL;
        log_debug(ZONE,"session exited externally");
        return;
    }

    log_debug(ZONE,"HELL %d\t%s",demon,xmlnode2str(p->x));
    switch(p->type)
    {
    case JPACKET_MESSAGE:
        fprintf(stderr,"m");
        break;
    case JPACKET_PRESENCE:
        fprintf(stderr,"p");
        break;
    case JPACKET_IQ:
        fprintf(stderr,"r");
        break;
    case JPACKET_S10N:
        fprintf(stderr,"s");
        break;

    }
    xmlnode_free(p->x);
}

void svc_hell_start(int demon)
{
    static char *server = NULL;
    xmlnode x, q;
    jid id;

    if(server == NULL)
        server = xmlnode_get_data(js_config("names/default"));

    log_debug(ZONE,"releasing demon %d",demon);

    x = xmlnode_new_tag("iq");
    xmlnode_put_attrib(x, "type", "set");
    q = xmlnode_insert_tag(x, "query");
    xmlnode_put_attrib(q, "xmlns", NS_REGISTER);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "username"),hell[demon].id,-1);
    xmlnode_insert_cdata(xmlnode_insert_tag(q, "password"),"a",1);

    js_service_prescreen(jpacket_new(x));

    id = jid_new(xmlnode_pool(x),server);
    jid_set(id,hell[demon].id,JID_USER);
    jid_set(id,"a",JID_RESOURCE);
    hell[demon].s = js_session_new(id,svc_hell_onSend,(void *)demon);

    fprintf(stderr," ");
    return;

    xmlnode_free(x);
    x = xmlnode_new_tag("presence");
    js_session_from(hell[demon].s,jpacket_new(x));
    x = jutil_iqnew(JPACKET__GET,NS_ROSTER);
    js_session_from(hell[demon].s,jpacket_new(x));

}

void *svc_hell_main(void *arg)
{
    int demon = (int)arg;
    int to, tst;
    xmlnode x, x2;

    srand(demon);
    while(1)
    {
        tst = (int) ((float)SVC_HELL_MAX*rand()/(RAND_MAX+1.0));
        pth_sleep((int)SVC_HELL_MAX*rand()/(RAND_MAX+1.0));

        if(svc_hell_sleep) continue;

        if(hell[demon].s == NULL)
            svc_hell_start(demon);

        tst = (int) ((float)100.0*rand()/(RAND_MAX+1.0));
        if(tst < 5)
        {
            js_session_end(hell[demon].s,"random chance");
            hell[demon].s = NULL;
            continue;
        }

        to = (int) ((float)SVC_HELL_MAX*rand()/(RAND_MAX+1.0));
        if(hell[to].s == NULL)
            continue;

        tst = (int) ((float)10.0*rand()/(RAND_MAX+1.0));
        switch(tst)
        {
        case 0: /* message */
        case 1:
        case 2:
        case 3:
        case 4:
            fprintf(stderr,"M");
            x = xmlnode_new_tag("message");
            xmlnode_put_attrib(x,"to",jid_full(hell[to].s->id));
            break;
        case 5: /* presence */
        case 6:
            /*continue;*/
            fprintf(stderr,"P");
            x = xmlnode_new_tag("presence");
            tst = (int) ((float)2.0*rand()/(RAND_MAX+1.0));
            if(tst)
                xmlnode_put_attrib(x,"type","unavailable");
            break;
        case 7: /* roster */
            /*continue;*/
            fprintf(stderr,"R");
            x = xmlnode_new_tag("iq");
            xmlnode_put_attrib(x,"type","set");
            x2 = xmlnode_insert_tag(x,"query");
            xmlnode_put_attrib(x2,"xmlns",NS_ROSTER);
            x2 = xmlnode_insert_tag(x2,"item");
            xmlnode_put_attrib(x2,"jid",jid_full(hell[to].s->uid));
            tst = (int) ((float)2.0*rand()/(RAND_MAX+1.0));
            if(tst)
                xmlnode_put_attrib(x2,"subscription","remove");
            else
                xmlnode_put_attrib(x,"subscription","none");
            break;
        case 8: /* s10n */
        case 9:
            /*continue;*/
            fprintf(stderr,"S");
            x = xmlnode_new_tag("presence");
            xmlnode_put_attrib(x,"to",jid_full(hell[to].s->uid));
            tst = (int) ((float)4.0*rand()/(RAND_MAX+1.0));
            switch(tst)
            {
            case 0:
                xmlnode_put_attrib(x,"type","subscribe");
                break;
            case 1:
                xmlnode_put_attrib(x,"type","unsubscribe");
                break;
            case 2:
                xmlnode_put_attrib(x,"type","subscribed");
                break;
            case 3:
                xmlnode_put_attrib(x,"type","unsubscribed");
                break;
            }
            break;
        }

        js_session_from(hell[demon].s,jpacket_new(x));
        if(svc_hell_total++ > SVC_HELL_ITER)
        {
            js_session_end(hell[demon].s,"done");
            hell[demon].s = NULL;
            return NULL;
        }
    }
}

void svc_hell(void)
{
    pth_attr_t attr;
    int i;

    attr = pth_attr_new();
    pth_attr_set(attr, PTH_ATTR_JOINABLE, FALSE);
    pth_attr_set(attr, PTH_ATTR_STACK_SIZE, etherx_stack_default);

    log_debug(ZONE,"svc_hell: init()");

    svc_hell_sleep = 0;

    for(i=0;i<SVC_HELL_MAX;i++)
    {
        sprintf(hell[i].id,"d%d",i);
        hell[i].s = NULL;
        pth_spawn(attr, svc_hell_main, (void *)i);
    }

    pth_attr_destroy(attr);
    fprintf(stderr,"\n");
}


