#include "svc_http.h"

/* SENDING: /message.*?to=user@server/resource&body=foo+bar optional: subject, type
 * FETCHING: /message.* optional: from=foo@bar to filter
 */

void svc_http_message_send(hrq rq)
{
    xmlnode x;

    x = xmlnode_new_tag("message");
    xmlnode_put_attrib(x,"type",xmlnode_get_attrib(rq->q,"type"));
    xmlnode_put_attrib(x,"to",xmlnode_get_attrib(rq->q,"to"));
    if(xmlnode_get_attrib(rq->q,"body") != NULL)
        xmlnode_insert_cdata(xmlnode_insert_tag(x,"body"),xmlnode_get_attrib(rq->q,"body"),-1);
    if(xmlnode_get_attrib(rq->q,"subject") != NULL)
        xmlnode_insert_cdata(xmlnode_insert_tag(x,"subject"),xmlnode_get_attrib(rq->q,"subject"),-1);

    log_debug(ZONE,"sending message %s",xmlnode2str(x));

    js_session_from(rq->s->s,jpacket_new(x));

    svc_http_session_reply(rq);
}

void svc_http_message(hrq rq)
{
    jpcache cur;
    jid filter;

    log_debug(ZONE,"/message request %s",xmlnode2str(rq->q));

    if(xmlnode_get_attrib(rq->q,"to") != NULL) return svc_http_message_send(rq);

    /* GET phase, scan the cache */

    /* check for filter if any */
    filter = jid_new(rq->p,xmlnode_get_attrib(rq->q,"from"));

    for(cur = rq->s->cache->next; cur != rq->s->cache; cur = cur->next)
    {
        if(cur->p->type != JPACKET_MESSAGE) continue;

        if(filter == NULL)
        {
            svc_http_cache_send(rq,cur);
            continue;
        }

        /* restrict to the server filter */
        if(cur->p->from == NULL || j_strcasecmp(cur->p->from->server,filter->server) != 0)
            continue;

        /* if the filter has a user, check that */
        if(filter->user != NULL && j_strcasecmp(cur->p->from->user,filter->user) != 0)
            continue;

        /* if the filter has a resource, check that */
        if(filter->resource != NULL && j_strcasecmp(cur->p->from->resource,filter->resource) != 0)
            continue;

        svc_http_cache_send(rq,cur);
    }

    svc_http_session_reply(rq);
}
