#include "svc_http.h"

/* HTTP interface for presence
 * setting
 *   /presence.*?type=available&status=foo+bar with optional show
 * getting
 *   /presence.* checks waiting cached ones
 *   /presence.*?from=user@server fetches their presence as stored by the server
 */

void svc_http_presence_set(hrq rq)
{
    xmlnode x;

    x = xmlnode_new_tag("presence");
    xmlnode_put_attrib(x,"type",xmlnode_get_attrib(rq->q,"type"));
    xmlnode_put_attrib(x,"to",xmlnode_get_attrib(rq->q,"to"));
    if(xmlnode_get_attrib(rq->q,"status") != NULL)
        xmlnode_insert_cdata(xmlnode_insert_tag(x,"status"),xmlnode_get_attrib(rq->q,"status"),-1);
    if(xmlnode_get_attrib(rq->q,"show") != NULL)
        xmlnode_insert_cdata(xmlnode_insert_tag(x,"show"),xmlnode_get_attrib(rq->q,"show"),-1);

    log_debug(ZONE,"presence packet %s",xmlnode2str(x));

    js_session_from(rq->s->s,jpacket_new(x));

    svc_http_session_reply(rq);
}

char *_urlbuild(hrq rq, char *type)
{
    spool sp;
    sp = spool_new(rq->p);
    spooler(sp,xmlnode_get_attrib(rq->q,"base"),xmlnode_get_attrib(rq->q,type),sp);
    return spool_print(sp);
}

/* get the presence for the gif thingie */
void svc_http_presence_gif(hrq rq)
{
    jid from;
    xmlnode pres;
    jpcache cur;
    char *url;

    from = jid_new(rq->p,xmlnode_get_attrib(rq->q,"from"));

    /* first, scan for waiting messages hack */
    url = xmlnode_get_attrib(rq->q,"message");
    if(from != NULL && url != NULL)
        for(cur = rq->s->cache->next; cur != rq->s->cache; cur = cur->next)
        {
            /* restrict to the server filter */
            if(cur->p->type != JPACKET_MESSAGE || cur->p->from == NULL || j_strcasecmp(cur->p->from->server,from->server) != 0)
                continue;

            /* if the filter has a user, check that */
            if(from->user != NULL && j_strcasecmp(cur->p->from->user,from->user) != 0)
                continue;

            /* if the filter has a resource, check that */
            if(from->resource != NULL && j_strcasecmp(cur->p->from->resource,from->resource) != 0)
                continue;

            /* we got here, so there's a message waiting, redirect to that */
            svc_http_redirect(rq,_urlbuild(rq,"message"));
            return;
        }

    /* pass off */
    pres = ppdb_primary(rq->s->s->u->p_cache,from);
    svc_http_presence_redirector(rq,pres);
}

/* based on the presence, redirect to a url in the query string */
void svc_http_presence_redirector(hrq rq, xmlnode pres)
{
    char *show;

    log_debug(ZONE,"presence redirector for %s",xmlnode2str(pres));

    if(pres == NULL || j_strcmp(xmlnode_get_attrib(pres,"type"),"unavailable") == 0) return svc_http_redirect(rq,_urlbuild(rq,"off"));

    show = xmlnode_get_tag_data(pres,"show");
    if(show == NULL || xmlnode_get_attrib(rq->q,show) == NULL) return svc_http_redirect(rq,_urlbuild(rq,"on"));

    svc_http_redirect(rq,_urlbuild(rq,show));
}

/* return presence from the ppdb */
void svc_http_presence_ppdb(hrq rq)
{
    xmlnode pnew;
    jid from;

    from = jid_new(rq->p,xmlnode_get_attrib(rq->q,"from"));

    while((pnew = ppdb_get(rq->s->s->u->p_cache,from)) != NULL)
        xmlnode_insert_tag_node(rq->body,pnew);

    svc_http_session_reply(rq);
}

void svc_http_presence(hrq rq)
{
    jpcache cur;

    log_debug(ZONE,"/presence request %s",xmlnode2str(rq->q));

    if(xmlnode_get_attrib(rq->q,"type") != NULL) return svc_http_presence_set(rq);

    if(strcmp(rq->ext,"gif") == 0) return svc_http_presence_gif(rq);

    if(xmlnode_get_attrib(rq->q,"from") != NULL) return svc_http_presence_ppdb(rq);

    /* request for waiting presence packets */
    for(cur = rq->s->cache->next; cur != rq->s->cache; cur = cur->next)
        svc_http_cache_send(rq,cur);

    svc_http_session_reply(rq);
}

