#include "svc_http.h"

/* this is where we send the request on to the right handler */
void svc_http_breakdance(hrq rq)
{
    /* top/public level services */
    if(strncmp(rq->path,"/public/presence",16) == 0) return svc_http_public_presence(rq);
    else if(strncmp(rq->path,"/public/message",15) == 0) return svc_http_public_message(rq);
    else if(strncmp(rq->path,"/XML",4) == 0) return svc_http_xml(rq);
    else if(strncmp(rq->path,"/RPC2",5) == 0) return svc_http_rpc(rq);
    else if(rq->ext == NULL) return svc_http_error(rq,404); /* rest of these require an extension */
    else if(strncmp(rq->path,"/register",9) == 0) return svc_http_register(rq);
    else if(strncmp(rq->path,"/session",8) == 0) return svc_http_session_new(rq);

    /* find the session for this request */
    svc_http_session(rq);

    /* make sure we have a session, if they use the sid just return error otherwise 401 em */
    if(rq->s == NULL)
    {
        if(xmlnode_get_attrib(rq->q,"sid") != NULL)
            return svc_http_session_reply(rq);
        else
            return svc_http_error(rq,401);
    }

    if(strncmp(rq->path,"/client",7) == 0) return svc_http_client(rq);
    else if(strncmp(rq->path,"/presence",9) == 0) return svc_http_presence(rq);
    else if(strncmp(rq->path,"/message",8) == 0) return svc_http_message(rq);
    else if(strncmp(rq->path,"/roster",7) == 0) return svc_http_roster(rq);
    else return svc_http_error(rq,404);
}

void svc_http_parse(hrq rq)
{
    char *qmark, *file, *dot, *user, *pass;

    if(j_strcmp(xmlnode_get_name(rq->x),"POST") == 0)
        svc_http_qs2xml(xmlnode_insert_tag(rq->x,"FORM"),xmlnode_get_data(rq->x));

    /* trim the :port from the host header */
    file = xmlnode_get_attrib(rq->x,"host");
    if(file != NULL && (dot = strstr(file,":")) != NULL)
        *dot = '\0';

    /* parse any query string */
    qmark = strstr(rq->path,"?");
    if(qmark != NULL)
    {
        *qmark = '\0';
        qmark++;
        if(xmlnode_get_tag(rq->x,"FORM") != NULL) /* reuse the FORM vars namespace */
            rq->q = xmlnode_dup_pool(rq->p,xmlnode_get_tag(rq->x,"FORM"));
        else
            rq->q = xmlnode_new_tag_pool(rq->p,"q");
        svc_http_qs2xml(rq->q,qmark);
    }

    /* decode any basic auth */
    if(j_strncmp(xmlnode_get_attrib(rq->x,"authorization"),"Basic ",6) == 0)
    {
        user = pass = NULL;
        user = strstr(xmlnode_get_attrib(rq->x,"authorization")," ");
        user++;
        str_b64decode(user);
        if(user != NULL)
            pass = strchr(user,':');
        if(pass != NULL)
        {
            *pass = '\0';
            pass++;
        }
        log_debug(ZONE,"Decoded %s / %s",user,pass);
        xmlnode_put_attrib(rq->x,"username",user);
        xmlnode_put_attrib(rq->x,"password",pass);
    }

    /* make the path friendlier */
    svc_http_strdecode(rq->path);
    file = strrchr(rq->path,'/');
    if(file != NULL && rq->file != rq->path)
    { /* file must be after a second slash */
        file++;
        rq->file = pstrdup(rq->p,file);
        dot = strrchr(rq->file,'.');
        if(dot != NULL)
        {
            *dot = '\0';
            dot++;
            rq->ext = dot;
        }
    }else{
        dot = strrchr(rq->path,'.');
        if(dot != NULL)
        {
            dot++;
            rq->ext = dot;
        }
    }

    log_debug(ZONE,"received request [%s] with query [%s] to %s",xmlnode2str(rq->x),xmlnode2str(rq->q),rq->path);
}

/* when the connection dies, clean up the mess */
void svc_http_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    hrq rq = (hrq)(ts->data);

    log_debug(ZONE,"svc_http_cleanup %X",rq);
}

void _scan_get(hrq rq)
{
    char *eol, *uri, *proto;

    /* get end of line or fail */
    eol = strstr(rq->buffer,"\r\n");
    if(eol == NULL) return;
    *eol = '\0';
    eol += 2;

    /* get the path */
    uri = strstr(rq->buffer," ");
    if(uri == NULL) return svc_http_error(rq,500);
    *uri = '\0';
    ++uri;

    /* get the protocol */
    proto = strstr(uri," ");
    if(proto == NULL) return svc_http_error(rq,500);
    *proto = '\0';
    ++proto;

    rq->x = xmlnode_new_tag_pool(rq->p,rq->buffer);
    rq->path = pstrdup(rq->p,uri);
    rq->buffer = eol;
    rq->phase = PHASE_HEAD;
}

void _scan_head(hrq rq)
{
    char *eol, *header, *str;

    for(eol = strstr(rq->buffer,"\r\n"); eol != NULL; eol = strstr(rq->buffer,"\r\n"))
    {
        if(eol == rq->buffer)
        {
            rq->buffer += 2; /* move past eol */
            if(j_strcmp(xmlnode_get_name(rq->x),"POST") == 0 && xmlnode_get_attrib(rq->x,"content-length") != NULL)
            {
                rq->clen = atoi(xmlnode_get_attrib(rq->x,"content-length"));
                rq->phase = PHASE_BODY;
            }else{
                rq->phase = PHASE_DONE;
            }
            return;
        }

        /* chop off */
        *eol = '\0';
        eol += 2;

        /* segment header/value */
        header = strstr(rq->buffer,": ");
        if(header == NULL) return svc_http_error(rq,500);
        *header = '\0';
        header += 2;

        /* lowercase the header names (IE SUCKS!) */
        for(str = rq->buffer; *str != '\0'; str++)
            *str = tolower(*str);
        xmlnode_put_attrib(rq->x,rq->buffer,header);
        rq->buffer = eol;
    }
}

void _scan_body(hrq rq)
{
    if(strlen(rq->buffer) < rq->clen) return;

    xmlnode_insert_cdata(rq->x,rq->buffer,rq->clen);
    rq->phase = PHASE_DONE;
}

/* read in the http request */
void svc_http_read(thread t, void *buffer, int bufsz)
{
    char *cur;
    tstream ts = (tstream)(t->data);
    hrq rq = (hrq)(ts->data);

    if(rq->phase == PHASE_INIT)
    {
        pool_cleanup(t->p, svc_http_cleanup, (void *)t);
        rq->phase = PHASE_GET;
        rq->buffer = pstrdup(rq->p,"");
    }

    if(bufsz <= 0) return;

    cur = pmalloc(rq->p,strlen(rq->buffer) + bufsz + 1);
    *cur = '\0';
    strcat(cur,rq->buffer);
    strcat(cur,buffer);
    rq->buffer = cur;

    log_debug(ZONE,"reading data [%s]",rq->buffer);

    if(rq->phase == PHASE_GET) _scan_get(rq);
    if(rq->phase == PHASE_HEAD) _scan_head(rq);
    if(rq->phase == PHASE_BODY) _scan_body(rq);

    if(rq->phase == PHASE_DONE)
    {
        svc_http_parse(rq);
        svc_http_breakdance(rq);
    }
}


/* new connection, setup our handlers */
void svc_http_connect(thread t, int sock, struct sockaddr_in sa)
{
    thread new;
    hrq rq;
    tstream ts;

    log_debug(ZONE,"svc_http connection established (fd: %d, ip: %s, port: %d)\n",
              sock, inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
    new = tstream_new(sock, inet_ntoa(sa.sin_addr), svc_http_read, NULL, NULL);
    ts = (tstream)(new->data);
    rq = pmalloc(new->p, sizeof(_hrq));
    memset(rq,0,sizeof(_hrq));
    rq->t = new;
    rq->p = t->p;
    rq->ip = ts->host;
    rq->re = xmlnode_new_tag_pool(t->p,"re");
    ts->data = (void *)rq;
}

/* startup routine for jserver */
void svc_http()
{
    log_debug(ZONE,"svc_http startup");
    js_service_listen("http", svc_http_connect);

    svc_http_public_init();
    svc_http_session_init();
}

void svc_http_qs2xml(xmlnode x, char *qs)
{
    char *key, *val, *nul;

    val = nul = qs + strlen(qs);
    key = qs;
    for(;qs[0] != '\0'; ++qs)
    {
        if(qs[0] == '+')
            qs[0] = ' ';
        if(qs[0] == '=')
        {
            qs[0] = '\0';
            val = qs + 1;
        }
        if(qs[0] == '&' || qs[1] == '\0')
        {
            if(qs[0] == '&')
                qs[0] = '\0';
            svc_http_strdecode(key);
            svc_http_strdecode(val);
            xmlnode_put_attrib(x,key,val);
            key = qs + 1;
            val = nul;
        }
    }
}

int _svc_http_dehex(char c)
{
    if(c >= '0' && c <= '9')
        return c - '0';
    if(c >= 'a' && c <= 'f')
        return c - 'a' + 10;
    if(c >= 'A' && c <= 'F')
        return c - 'A' + 10;
    return 0;
}

/* decode the uri hex junk */
void svc_http_strdecode(char *from)
{
    char *to;

    to = from;
    for(;*from != '\0'; ++to, ++from)
    {
        if(from[0] == '%' && strchr("0123456789abcdefABCDEF", from[1]) != NULL && strchr("0123456789abcdefABCDEF", from[2]) != NULL)
        {
            *to = _svc_http_dehex(from[1]) * 16 + _svc_http_dehex(from[2]);
            from += 2;
        }else{
            *to = *from;
        }
    }
    *to = '\0';
}

void svc_http_error(hrq rq, int err)
{
    rq->re_code = err;
    svc_http_reply(rq);
}

void svc_http_redirect(hrq rq, char *url)
{
    rq->re_code = 302;
    xmlnode_put_attrib(rq->re,"Location",url);
    svc_http_reply(rq);
}

void svc_http_reply_mime(hrq rq, int type)
{
    switch(type)
    {
    case MIME_HTML:
        xmlnode_put_attrib(rq->re,"Content-type","text/html");
        break;
    case MIME_JS:
        xmlnode_put_attrib(rq->re,"Content-type","application/x-javascript");
        break;
    case MIME_XML:
        xmlnode_put_attrib(rq->re,"Content-type","text/xml");
        break;
    case MIME_GIF:
        xmlnode_put_attrib(rq->re,"Content-type","image/gif");
        break;
    case MIME_TXT:
        xmlnode_put_attrib(rq->re,"Content-type","text/plain");
        break;
    }
}

void svc_http_reply_append(hrq rq, ...)
{
    va_list ap;
    char *arg = NULL;
    if(rq == NULL)
        return;

    va_start(ap, rq);
    /* loop till we hit our end flag, the first arg */
    while(1)
    {
        arg = va_arg(ap,char *);
        if((int)arg == (int)rq)
            break;
        else if(arg != NULL)
            xmlnode_insert_cdata(rq->re,arg,strlen(arg));
    }
    va_end(ap);
}

/* sends the reply, uses rq->re */
void svc_http_reply(hrq rq)
{
    spool s;
    char *body;
    xmlnode cur;
    char len[10];

    s = spool_new(rq->p);
    spool_add(s,"HTTP/1.0 ");
    switch(rq->re_code)
    {
    case 302:
        spool_add(s,"302 Temporary Redirect\r\n");
        svc_http_reply_mime(rq,MIME_HTML);
        svc_http_reply_append(rq,"Redirecting",rq);
        break;
    case 401:
        spool_add(s,"401 Unauthorized\r\n");
        spool_add(s,"WWW-authenticate: basic realm=\"Jabber\"\r\n");
        svc_http_reply_mime(rq,MIME_HTML);
        svc_http_reply_append(rq,"Unauthorized",rq);
        break;
    case 404:
        spool_add(s,"404 Not Found\r\n");
        svc_http_reply_mime(rq,MIME_HTML);
        svc_http_reply_append(rq,"<H1>Not Found</H1>",rq);
        break;
    case 500:
        spool_add(s,"500 Internal Error\r\n");
        svc_http_reply_mime(rq,MIME_HTML);
        svc_http_reply_append(rq,"<H1>Internal Server Error</H1>",rq);
        break;
    default:
        spool_add(s,"200 OK\r\n");
    }

    spool_add(s,"Server: jabber/svc_http\r\n");
    spool_add(s,"Expires: Fri, 10 Oct 1997 10:10:10 GMT\r\nPragma: no-cache\r\nCache-control: private\r\n");
    spool_add(s,"Connection: close\r\n");

    for(cur = xmlnode_get_firstattrib(rq->re); cur != NULL; cur = xmlnode_get_nextsibling(cur))
        spooler(s,xmlnode_get_name(cur),": ",xmlnode_get_data(cur),"\r\n",s);

    body = xmlnode_get_data(rq->re);
    if(body != NULL)
    {
        sprintf(len,"%d",strlen(body));
        spooler(s,"Content-length: ",len,"\r\n\r\n",s);
        spool_add(s,body);
    }else{
        spool_add(s,"Content-length: 0\r\n\r\n");
    }

    body = spool_print(s);
    log_debug(ZONE,"HTTP response [%s]",body);
    tstream_write(rq->t, body, -1);
    tstream_close(rq->t);
}

