#include "jserver.h"

#define PHASE_INIT 0
#define PHASE_GET 1
#define PHASE_HEAD 2
#define PHASE_BODY 3
#define PHASE_DONE 4

#define MIME_HTML 0
#define MIME_JS 1
#define MIME_XML 2
#define MIME_GIF 3
#define MIME_TXT 4

#define HTTP_MINWAIT 120

/* cache ring */
typedef struct jpcache_struct
{
    jpacket p;
    int sent;
    struct jpcache_struct *next, *prev;
} *jpcache, _jpcache;

typedef struct hrq_struct *hrq, _hrq;

/* session ring */
typedef struct hs_struct
{
    session s;
    char *sid, *user, *pass;
    ppdb pproxy;
    jpcache cache;
    int last;
    int timeout;
    struct hs_struct *next, *prev;
    hrq rt; /* realtime-requests for this session */
} *hs, _hs;

/* http request data */
struct hrq_struct
{
    int phase, clen, re_code, re_flag;
    char *path, *file, *ext;
    char *ip;
    char *buffer;
    hs s;
    jid id;
    thread t;
    xmlnode x, re, q, body;
    pool p;
    struct hrq_struct *next;
};

void svc_http_qs2xml(xmlnode x, char *qs);
void svc_http_strdecode(char *from);
void svc_http_error(hrq rq, int err);
void svc_http_redirect(hrq rq, char *url);
void svc_http_reply_mime(hrq rq, int type);
void svc_http_reply_append(hrq rq, ...);
void svc_http_reply(hrq rq);

void svc_http_session_new(hrq rq);
void svc_http_session_end(hs h);
void svc_http_session(hrq rq);
void svc_http_session_reply(hrq rq);
void svc_http_session_init(void);

void svc_http_public_init(void);
void svc_http_public_presence(hrq rq);
void svc_http_public_message(hrq rq);

void svc_http_presence(hrq rq);
void svc_http_presence_redirector(hrq rq, xmlnode pres);
void svc_http_message(hrq rq);
void svc_http_roster(hrq rq);
void svc_http_client_in(session s, jpacket p, void *arg);
void svc_http_client(hrq rq);
void svc_http_xml(hrq rq);
void svc_http_rpc(hrq rq);

void svc_http_cache(hs s, jpacket p);
void svc_http_cache_send(hrq rq, jpcache cur);

void svc_http_hrt(hrq rq);
void svc_http_register(hrq rq);
