#include "svc_irc.h"

/* IN: Jabber XML --> IRC */

/* util for ctcp */
char *svc_irc_2ctcp(pool p, char *cmd, char *data)
{
    int i;
    char *body;

    if(p == NULL || body == NULL || cmd == NULL)
        return NULL;

    /* translate to CTCP */
    body = pmalloc(p,strlen(data) + strlen(cmd) + 5);
    body[0] = 1;
    body[1] = '\0';
    strcat(body,cmd);
    strcat(body," ");
    strcat(body,data);
    i = strlen(body);
    body[i] = 1;
    body[i+1] = '\0';

    return body;
}

/* send a message to the client */
void svc_irc_inmsg(ircdata d, jpacket p)
{
    char *body;

    body = xmlnode_get_tag_data(p->x,"body");
    if(body == NULL)
        body = pstrdup(p->p," ");

    if(jpacket_subtype(p) == JPACKET__ERROR)
    {
        svc_irc_line(d, NULL, "NOTICE", xmlnode_get_tag_data(p->x,"error"), 6, d->nick, ":Your message [",body,"] to ",jid_full(p->from),"failed");
        return;
    }

    /* translate /me to CTCP ACTION */
    if(strncmp(body,"/me ",4) == 0)
        body = svc_irc_2ctcp(p->p,"ACTION",body + 4);

    if(xmlnode_get_attrib(svc_irc_item(d,p->from),"subscription") != NULL && jpacket_subtype(p) != JPACKET__CHAT)
        svc_irc_line(d, svc_irc_nick(d,p->from), "PRIVMSG", body, 1, ROOM);
    else
        svc_irc_line(d, svc_irc_nick(d,p->from), "PRIVMSG", body, 1, d->nick);

}

/* send presence to the client */
void svc_irc_inpres(ircdata d, jpacket p)
{
    xmlnode old_flag, item, new_flag;
    char *status, *show, msg[400], *qm, *body;
    jid uid;

    /* create a nice message to the user */
    status = xmlnode_get_tag_data(p->x,"status");
    show = xmlnode_get_tag_data(p->x,"show");
    if(status != NULL || show != NULL)
    {
        msg[0] = '\0';
        if(p->from->resource != NULL)
        {
            strcat(msg,"(");
            qm = strchr(p->from->resource,'?');
            if(qm != NULL)
                strncat(msg,p->from->resource,50);
            else
                strncat(msg,p->from->resource,qm - p->from->resource);
            strcat(msg,") ");
        }
        strcat(msg,"is ");
        if(jpacket_subtype(p) == JPACKET__UNAVAILABLE)
            strcat(msg,"offline");
        else if(show == NULL)
            strcat(msg,"online");
        else
            strncat(msg,show,5);

        if(status != NULL)
        {
            strcat(msg,": ");
            strncat(msg,status,200);
        }

        body = svc_irc_2ctcp(p->p,"ACTION ",msg);
        svc_irc_line(d, svc_irc_nick(d,p->from), "PRIVMSG", body, 1, ROOM);
    }

    if(p->from->user != NULL)
    {
        uid = jid_new(p->p,jid_full(p->from));
        jid_set(uid,NULL,JID_RESOURCE);
    }else{
        uid = p->from;
    }

    item = svc_irc_item(d, p->from);

    old_flag = ppdb_primary(d->pres,uid);
    d->pres = ppdb_insert(d->pres,p->from,p->x);
    new_flag = ppdb_primary(d->pres,uid);

    if(item == NULL || (new_flag == NULL && old_flag == NULL)) return;

    if(jpacket_subtype(p) == JPACKET__AVAILABLE && old_flag == NULL)
        svc_irc_line(d, "presence", "MODE", NULL, 3, ROOM, "+o", xmlnode_get_attrib(item,"name"));
    else if(old_flag != NULL && new_flag == NULL)
        svc_irc_line(d, "presence", "MODE", NULL, 3, ROOM, "-o", xmlnode_get_attrib(item,"name"));
}

/* send a subscription request to the client */
void svc_irc_ins10n(ircdata d, jpacket p)
{
    svc_irc_line(d, "nickserv", "NOTICE", NULL, 7, d->nick, ":Presence", xmlnode_get_attrib(p->x,"type"), "request from", jid_full(p->from), ":", xmlnode_get_tag_data(p->x,"status"));
}

/* manage the roster for the client */
void svc_irc_inroster(ircdata d, jpacket p)
{
    xmlnode cur, inew, iold;
    jid id;
    spool members;
    char *ids;

    if(jpacket_subtype(p) == JPACKET__RESULT)
    { /* got our roster */
        d->roster = xmlnode_dup_pool(d->t->p, p->iq);
        members = spool_new(p->p);
        spooler(members, "@", d->nick, " ", members);
        for(cur = xmlnode_get_firstchild(d->roster); cur != NULL; cur = xmlnode_get_nextsibling(cur))
        {
            if((ids = xmlnode_get_attrib(cur,"jid")) != NULL)
            {
                svc_irc_nick(d, jid_new(p->p,ids)); /* makes sure name="" has a unique nick */
                if(j_strcmp(xmlnode_get_attrib(cur,"subscription"),"both") == 0 || j_strcmp(xmlnode_get_attrib(cur,"subscription"),"to") == 0)
                    spooler(members, "+", xmlnode_get_attrib(cur,"name"), " ", members);
                else
                    spooler(members, xmlnode_get_attrib(cur,"name"), " ", members);
            }
        }
        svc_irc_line(d,NULL,"353",spool_print(members),3,d->nick,"=",ROOM);

        if(d->whoflag)
        {
            svc_irc_who(d, ROOM);
            d->whoflag = 0;
        }
        return;
    }

    /* handle roster pushes */
    if(jpacket_subtype(p) != JPACKET__SET)
        return;

    inew = xmlnode_get_firstchild(p->iq);
    id = jid_new(p->p,xmlnode_get_attrib(inew,"jid"));
    iold = svc_irc_item(d,id);

    xmlnode_insert_tag_node(d->roster,inew);
    svc_irc_nick(d,id); /* make sure it has a nick */

    if(xmlnode_get_attrib(iold,"subscription") == NULL)
        svc_irc_line(d,xmlnode_get_attrib(inew,"name"),"JOIN",NULL,1,ROOM);
    if(xmlnode_get_attrib(iold,"name") != NULL && xmlnode_get_attrib(inew,"name") != NULL && j_strcmp(xmlnode_get_attrib(iold,"name"),xmlnode_get_attrib(inew,"name")) != 0)
        svc_irc_line(d,xmlnode_get_attrib(iold,"name"),"NICK",NULL,1,xmlnode_get_attrib(inew,"name"));
    xmlnode_hide(iold);
}

/* manage incoming iq packet */
void svc_irc_inagents(ircdata d, jpacket p)
{
    xmlnode query,agent;
    query=xmlnode_get_tag(p->x,"query");
    if(query==NULL) return;
    agent=xmlnode_get_firstchild(query);
    svc_irc_line(d,"agents","NOTICE" ,"-------------------------------------------------------",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","This server has the following agents available:",1,d->nick);
    svc_irc_line(d,"agents","NOTICE" ,"-------------------------------------------------------",1,d->nick);
    while(agent!=NULL)
    {
        if(xmlnode_get_tag(agent,"register")!=NULL)
        { /* registerable agent */
            svc_irc_line(d,"agents","PRIVMSG",xmlnode_get_attrib(agent,"jid"),1,d->nick);
        }
        agent=xmlnode_get_nextsibling(agent);
    }
    svc_irc_line(d,"agents","NOTICE" ,"-------------------------------------------------------",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","To register for an agent, simply type in the id",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","listed above into this window, and I will help",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","you register with that agent.",1,d->nick);
    svc_irc_line(d,"agents","NOTICE" ,"-------------------------------------------------------",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG" ,"Enter a JID to register with:",1,d->nick);
    d->agent_phase=APHASE_REG_SET;
    if(d->agent_fields!=NULL&&d->agent_fields->p!=NULL)
    {
        pool_free(d->agent_fields->p);
        d->key=d->agent_jid=NULL;
        d->agent_fields=d->next_field=NULL;
    }
}

void svc_irc_inregister(ircdata d, jpacket p)
{
    pool new;
    spool sp;
    xmlnode query,field;
    if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"error")==0)
    {
        svc_irc_line(d,"agents","PRIVMSG","Failed to connect with that agent. try another.",1,d->nick);
        return;
    }
    else if(d->agent_fields!=NULL&&d->agent_fields->p!=NULL)
    {
        svc_irc_line(d,"agents","PRIVMSG","Registration successfull",1,d->nick);
        pool_free(d->agent_fields->p);
        d->agent_fields=d->next_field=NULL;
        d->agent_jid=d->key=NULL;
        return;
    }
    query=xmlnode_get_tag(p->x,"query");
    if(query==NULL) return;
    field=xmlnode_get_firstchild(query);
    if(field==NULL) return;
    new=pool_new();
    d->agent_jid=pstrdup(new,xmlnode_get_attrib(p->x,"from"));
    d->agent_fields=pmalloc(new,sizeof(_fields));
    d->agent_fields->p=new;
    d->agent_fields->name=NULL;
    d->agent_fields->data=NULL;
    d->agent_fields->next=NULL;
    d->agent_fields->prev=NULL;
    d->next_field=d->agent_fields;
    while(field!=NULL)
    {
        if(xmlnode_get_name(field)==NULL)
        {
            field=xmlnode_get_nextsibling(field);
            continue;
        }
        if(j_strcmp(xmlnode_get_name(field),"instructions")==0)
        {
            svc_irc_line(d,"agents","NOTICE" ,"-------------------------------------------------------",1,d->nick);
            svc_irc_line(d,"agents","PRIVMSG",xmlnode_get_data(field),1,d->nick);
            field=xmlnode_get_nextsibling(field);
            continue;
        }
        else if(j_strcmp(xmlnode_get_name(field),"registered")==0)
        {
            field=xmlnode_get_nextsibling(field);
            continue;
        }
        if(j_strcmp(xmlnode_get_name(field),"key")==0)
        {
            d->key=pstrdup(d->t->p,xmlnode_get_data(field));
            field=xmlnode_get_nextsibling(field);
            continue;
        }

        if(d->next_field->name!=NULL)
        {
            d->next_field->next=pmalloc(d->next_field->p,sizeof(_fields));
            d->next_field->next->p=d->next_field->p;
            d->next_field->next->prev=d->next_field;
            d->next_field=d->next_field->next;
            d->next_field->next=NULL;
            d->next_field->name=NULL;
            d->next_field->data=NULL;
        }
        d->next_field->name=pstrdup(d->next_field->p,xmlnode_get_name(field));
        d->next_field->data=pstrdup(d->next_field->p,xmlnode_get_data(field));
        field=xmlnode_get_nextsibling(field);
    }
    d->next_field=d->agent_fields;
    /* display the first line */
    svc_irc_line(d,"agents","NOTICE" ,"-------------------------------------------------------",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","I will display a field, and you will type the data for that field.",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","When the last field is filled in, the registration will be sent.",1,d->nick);
    svc_irc_line(d,"agents","NOTICE" ,"-------------------------------------------------------",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","Type 'back' for the previous item",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","Type 'skip' to skip this item",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","Type 'cancel' to kill this request, or",1,d->nick);
    svc_irc_line(d,"agents","PRIVMSG","Type 'remove' to remove an existing registration",1,d->nick);
    svc_irc_line(d,"agents","NOTICE" ,"-------------------------------------------------------",1,d->nick);
    sp=spool_new(new);
    spooler(sp,"Field: --=>",d->next_field->name,"<=-- Current Data: ",d->next_field->data,sp);
    svc_irc_line(d,"agents","PRIVMSG",spool_print(sp),1,d->nick);
    svc_irc_line(d,"agents","NOTICE" ,"-------------------------------------------------------",1,d->nick);
    svc_irc_line(d,"agents","NOTICE" ,"Enter the new data for this item:",1,d->nick);

}

/* incoming data from the session to the client */
void svc_irc_in(session s, jpacket p, void *arg)
{
    ircdata d = (ircdata)arg;

    /* session ended (XXX improve) */
    if(p == NULL)
    {
        if(d != NULL)
        {
            d->s = NULL;
            svc_irc_line(d,NULL,"NOTICE","Your session has been terminated by the server",0);
            tstream_close(d->t);
        }
        return;
    }

    log_debug(ZONE,"sent %s",xmlnode2str(p->x));

    if(d->debug)
        svc_irc_line(d,"debug","PRIVMSG",xmlnode2str(p->x),1,d->nick);

    /* send to client */
    switch(p->type)
    {
    case JPACKET_MESSAGE:
        svc_irc_inmsg(d,p);
        break;
    case JPACKET_PRESENCE:
        svc_irc_inpres(d,p);
        break;
    case JPACKET_S10N:
        svc_irc_ins10n(d,p);
        break;
    case JPACKET_IQ:
        if(NSCHECK(p->iq,NS_ROSTER))
            svc_irc_inroster(d,p);
        else if(NSCHECK(p->iq,NS_AGENTS))
            svc_irc_inagents(d,p);
        else if(NSCHECK(p->iq,NS_REGISTER))
            svc_irc_inregister(d,p);
        break;
    }

    xmlnode_free(p->x);
}
